# -*- coding: utf-8 -*-
"""
Created on Mon Jun 24 14:27:34 2019

@author: asano
"""


# 4.4節　いくつかの改善
import numpy as np
from scipy.stats import pearsonr

def logistic(x):
    return 1 / (1 + np.exp(-x))

# 初期値と観測データ
    
W1 = np.array([[0.603, 0.107, 0], [1.811, -0.307, 0],[-0.165,-1.108,0]])
W2 = np.array([[1.094, -0.494,-0.239,0], [-0.253,-0.945,0.212,0]])
X = np.array([[9,6,3,3,1,5,2,1,1,2], [3,3,0,1,1,4,5,0,8,2]])
T = np.array([[0.7,0.3,0.0,0.4,0.6,0.4,0.3,0.5,0.4,0.3],
              [0.8,0.9,0.8,0.7,0.6,0.4,0.3,0.8,0.7,0.0]])

N = 10 #分析ケース数、しばしば消費者の数
L = 3  #第２層（隠れ層）のユニット数
vec1 = np.ones((1,N))
Xex  = np.vstack([X,vec1])
vecL = np.ones((L,1))
Imatrix = vecL @ vec1    #1を要素としたL行N列の行列


# ここからイテレーションスタート
losshist = np.zeros((0,2))   #イテレーション前に記録の初期化
for ite in range(5001):
    
# フォワードステップ

    S = W1 @ Xex
    Z = logistic(S)
    Zex= np.vstack([Z,vec1 ])
    Y = W2 @ Zex
    Dif = Y - T
    
# イテレーション100回ごとに損失関数を記録
    if ite % 100 == 0:
        esum = (Dif.T @ Dif).trace()/2
        losshist=np.vstack((losshist,np.array([ite,esum])))
        print ( "ite= %d loss = %.3f" %(ite, esum))

# バックワードステップ

    DW2 = Dif @ Zex.T
    W2b = np.delete(W2,L,axis=1)
    DW1 = ((W2b.T @ Dif)*Z * (Imatrix - Z)) @ Xex.T

# パラメータ更新 ステップ幅は0.01　
    W1 = W1 - 0.01 * DW1
    W2 = W2 - 0.01 * DW2

# イテレーション終了後の適合度
print('イテレーション終了')
print('ite=',ite,'esum=',esum)
yvec = Y.flatten()
tvec = T.flatten()
r,p = pearsonr(yvec,tvec)   #無相関検定のP値
print('相関係数 r:  ',r)
print('疑似指標 r2: ',r**2)

# イテレーション後に学習曲線を描画
import matplotlib.pyplot as plt
plt.plot(losshist[0:,0],losshist[0:,1])
plt.show()





