# -*- coding: utf-8 -*-
"""
Created on Wed May 20 08:52:24 2020

@author: asano
"""

import numpy as np
import pandas as pd
np.set_printoptions(precision=3,suppress=True )
"""
説明変数データと教師データを同時に入力したい
教師データはカテゴリー番号のまま入力しプログラム内で
ダミー変数に変換する
データ行列は外部ファイルを転置することがふつうである
"""

df = pd.read_csv('branddata300.csv',header=None)
brand3 = df.iloc[0:300,4].astype(str)
Tdata = pd.get_dummies(brand3)
Td = np.array(Tdata)
T = Td.T

Xdata = df.iloc[0:300,0:4].values
X = Xdata.T

# 関数の定義
def softmax(x):
    mx = x.max(axis=0)
    xd = x - mx
    u =np.exp(xd)
    return (u / u.sum(axis=0))

def cross_e(t,p):
    delta = 1e-7
    return -np.mean(np.sum(t * np.log(p + delta),axis=0))

def ReLU(x):
    return np.maximum(0,x)

def step(x):
    return 1.0 * (x > 0)

N = 300  #分析ケース数、しばしば消費者の数
M = 3    #予測したいグループ数
L = 10   #隠れ層ユニット数
K = 4    #説明変数の数

vec1 = np.ones((1,N))   #1を要素としたＮ次のベクトル
Xex  = np.vstack([X,vec1])

# テストごとに初期値をリセット 
crosshist = np.zeros((0,2))   #★イテレーション前に記録を初期化する

# ウェイト初期値
# 初期値をランダムに変更したい場合は下記の4行を実行
W1 = np.random.rand(L,K+1) #新規乱数
W2 = np.random.rand(M,L+1) #新規乱数
W1 = W1/np.sqrt(K/2)
W2 = W2/np.sqrt(L/2)    

# ここからイテレーションスタート
for ite in range(1000):

# フォワードステップ

    S = W1 @ Xex
    Z = ReLU(S)
    Zex= np.vstack([Z,vec1 ])
    Y = W2 @ Zex
    P = softmax(Y)
    Dif = P - T
    cross = cross_e(T,P)   #損失関数

# イテレーションごとに損失関数を記録
    if ite % 100 == 0:
        cross = cross_e(T,P)   #損失関数
        crosshist=np.vstack((crosshist,np.array([ite,cross])))
        print ( "ite= %d cross = %.3f" %(ite, cross))

# バックワードステップ

    DW2 = Dif @ Zex.T
    W2b = W2[ : , :L]
    DW1 = ((W2b.T @ Dif)*step(S)) @ Xex.T

# パラメータ更新 ステップ幅は0.001　
    W1 = W1 - 0.001 * DW1
    W2 = W2 - 0.001 * DW2

# イテレーション終了後の適合度
print('イテレーション終了')
print('ite=',ite,'交差エントロピー=',cross)
# ウェイトの最終推定値
print('W1=',W1)
print('W2=',W2)

# イテレーション後に学習曲線を描画
import matplotlib.pyplot as plt
plt.plot(crosshist[0:,0],crosshist[0:,1])
plt.show()


"""
6.2節　予測は適中するか
アーグマックス関数でグループ番号を出力する
"""
group = np.argmax(Y, axis=0)+1
# 教師データをGとする
brand = df.iloc[0:300,4].values
accuracy = np.sum(brand == group)/N
print(accuracy)
# 予測グループgroupと真のbrandの間でクロス集計
import pandas as pd
print(pd.crosstab(group, brand))

